import * as kysely from 'kysely';
import { Kysely, TableMetadata, SelectQueryBuilder } from 'kysely';
import { ExtractTableAlias } from 'kysely/dist/esm/parser/table-parser';
import { z } from 'zod';
import { Connection, Queue } from 'plainjob';
import * as hono from 'hono';
import { ErrorHandler } from 'hono';
export { DB } from './db.js';
import * as hono_jsx_jsx_dev_runtime from 'hono/jsx/jsx-dev-runtime';
import Toastify from 'toastify-js';
import { Session } from 'hono-sessions';

type PickNullable<T> = {
    [P in keyof T as null extends T[P] ? P : never]: T[P];
};
type PickNotNullable<T> = {
    [P in keyof T as null extends T[P] ? never : P]: T[P];
};
type OptionalNullable<T> = {
    [K in keyof PickNullable<T>]?: T[K];
} & {
    [K in keyof PickNotNullable<T>]: T[K];
};
type InsertableEntity<DB, EntityName extends keyof DB & string> = OptionalNullable<Omit<DB[EntityName], "id" | "createdAt" | "updatedAt">> & {
    id?: string;
    createdAt?: number;
    updatedAt?: number;
};
declare class Entity<DB, EntityName extends keyof DB & string> {
    private db;
    private name;
    private id;
    private timestamp;
    zod: z.ZodType<DB[EntityName]>;
    createdAt: boolean;
    updatedAt: boolean;
    constructor(db: Kysely<DB>, name: EntityName, tables: TableMetadata[], _zod?: z.ZodType<DB[EntityName]>, id?: () => string, timestamp?: () => number);
    query(): SelectQueryBuilder<DB, ExtractTableAlias<DB, EntityName>, {}>;
    createMany(data: Array<InsertableEntity<DB, EntityName>>): Promise<DB[EntityName][]>;
    create(data: InsertableEntity<DB, EntityName>): Promise<DB[EntityName]>;
    all<K extends keyof DB[EntityName]>(filter?: Partial<Pick<DB[EntityName], K>>, pagination?: {
        limit?: number;
        offset?: number;
    }): Promise<DB[EntityName][]>;
    get<K extends keyof DB[EntityName]>(filter?: Partial<Pick<DB[EntityName], K>>): Promise<DB[EntityName] | undefined>;
    update(data: OptionalNullable<Omit<DB[EntityName], "createdAt" | "updatedAt">> & {
        id: string;
        updatedAt?: number;
    }): Promise<DB[EntityName]>;
    updateMany(data: Array<OptionalNullable<Omit<DB[EntityName], "createdAt" | "updatedAt">> & {
        id: string;
        updatedAt?: number;
    }>): Promise<DB[EntityName][]>;
    delete<K extends keyof DB[EntityName]>(filter: Partial<Pick<DB[EntityName], K>>): Promise<kysely.DeleteResult[]>;
}
type EntityStore<DB> = {
    <T extends keyof DB & string>(key: T): Entity<DB, T>;
    db: Kysely<DB>;
    opts?: EntityStoreOptions<DB>;
};
type EntityStoreOptions<DB> = {
    [K in keyof DB]: {
        id?: () => string;
        timestamp?: () => number;
        zod?: z.ZodType<DB[K]>;
    };
};
declare function store<DB>(db: Kysely<DB>, opts?: EntityStoreOptions<DB>): Promise<EntityStore<DB>>;

declare function migrate(db: Kysely<any>): (...fns: ((db: Kysely<any>) => Promise<void>)[]) => Promise<void>;
/**l
 * Run a function in a database transaction.
 * The transaction is automatically rolled back, even if the function doesn't throw an error.
 * Use during testing, to keep test cases isolated from each other.
 * */
declare function rollback<T>(db: Kysely<T>, fn: (db: Kysely<T>) => Promise<void>): Promise<void>;

declare function dev(): boolean;
declare function prod(): boolean;
declare function test(): boolean;

type Job<T> = {
    name: string;
    run: ({ data }: {
        data: T;
    }) => Promise<void>;
};
declare function job<T>(opts: {
    name: string;
    run: ({ data }: {
        data: T;
    }) => Promise<void>;
}): Job<T>;
type Schedule = {
    name: string;
    cron: string;
    run: () => Promise<void>;
};
declare function schedule(opts: {
    name: string;
    cron: string;
    run: () => Promise<void>;
}): Schedule;
declare function queue(opts: {
    connection: Connection;
}): Queue;
declare function work(queue: Queue, jobs: Record<string, Job<any>>, schedules: Record<string, Schedule>): Promise<void>;
declare function perform<T>(queue: Queue, job: Job<T>, data: T): Promise<{
    id: number;
} | {
    id: string;
}>;

declare function cacheFormData(): hono.MiddlewareHandler<any, string, {}>;

type ToastProps = {
    toast: unknown;
} & Toastify.Options;
declare function Toast(props: ToastProps): hono_jsx_jsx_dev_runtime.JSX.Element | null;

declare function protect<User>({ signin, getUser, }: {
    signin: string;
    getUser: (id: string) => Promise<User | undefined>;
}): hono.MiddlewareHandler<{
    Variables: {
        user: User;
        session?: Session;
    };
}, string, {}>;
declare function signin<User>({ getUser, protected: path, }: {
    getUser: (id: string) => Promise<User | undefined>;
    protected: string;
}): hono.MiddlewareHandler<{
    Variables: {
        user: User;
        session?: Session;
    };
}, string, {}>;

declare function error(): ErrorHandler;

export { Toast, cacheFormData, dev, error, job, migrate, perform, prod, protect, queue, rollback, schedule, signin, store, test, work };
